;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Dan Bothell 
;;; Address     : Carnegie Mellon University
;;;             : Psychology Department
;;;             : Pittsburgh,PA 15213-3890
;;;             : db30+@andrew.cmu.edu
;;; 
;;; Copyright   : (c)2002-2004 Dan Bothell
;;; Availability: Covered by the GNU LGPL, see LGPL.txt
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : graphic-trace.lisp
;;; Version     : 1.5
;;; 
;;; Description : NO system dependent code.
;;;             : This file contains the functions and variables 
;;;             : for recording and displaying the Graphic (PERT chart like)
;;;             : trace in the environment.
;;; Bugs        : 
;;; 
;;; Todo        : Clean up how and what it sends because the current full
;;;             : handshaking and pure Tcl/Tk code mechanism is very slow.
;;; 
;;; ----- History -----
;;;
;;; 10/01/2002  Dan
;;;             : Added this header.
;;; 4/22/2004   Dan [1.5]
;;;             : Added the license info.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(in-package :cl-user)

;#+(and :allegro-ide (not :ACTR-ENV-ALONE)) (in-package :cg-user)

(defstruct trace-block name line start-time stop-time in-lines out-lines details)

(defvar *graphic-trace* nil)


(defparameter *gt-display-items* nil)
(defparameter *gt-initiated* nil)

(defun clear-trace-record (event)
  (declare (ignore event))
  (setf *graphic-trace* nil)
  (setf *gt-display-items* nil)
  (setf *gt-initiated* nil))

(defun graphic-trace-recorder (event)
  (if (or (eq :reset *which-hook-fn*)
          (subtypep (type-of event) 'environment-handler))
      ;; when it's reset or just started clear the current trace
      
        (setf *graphic-trace* nil) 
        
      (when (subtypep (type-of event) 'input-queue-entry)
        (push
         (list event
               (if (eq (destination event) :cognition)
                   (cond ((equal (car (params event)) 'execute-rhs)
                          (list (production-name (instantiation-production *instantiation*))
                                (copy-tree (production-text (instantiation-production *instantiation*)))))
                         ((equal (car (params event)) 'complete-retrieval)
                          (let ((w (wme-name (cdr *retrieval-scheduler*))))
                            (cons w (with-output-to-string (s)
                                      (let ((*standard-output* s))
                                        (wm-fct (list w)))))))
                         (t nil))
                   nil))
         *graphic-trace*)))
  nil)




(defun graphic-trace-return (event)
  (declare (ignore event))
  (if (null *gt-display-items*)
      (if *gt-initiated*
          (progn 
            (setf *gt-initiated* nil)
            (list 'finished 0))
          (progn
            (setf *gt-initiated* t)
            (let ((val (copy-tree (graphic-trace-builder))))
              (setf *gt-display-items* (butlast val))
              (car (last val)))))
      (pop *gt-display-items*)))
  

(defun graphic-trace-builder ()
  (let ((items (list (list 'size 0 (pm-get-time ))))
        (cognition nil)
        (vision nil)
        (motor nil)
        ;(device nil)
        (retrieval nil)
        ;(things nil)
        (last-ret-request nil)
        (last-goal nil)
        (last-ret nil)
        ;(last-mot nil)
        (last-vis-loc nil)
        (last-vis nil)
        )
    ;(pprint *graphic-trace*)
    (dolist (x *graphic-trace*)
      (let* ((q (car x))
             (details (cadr x))
             (dest (destination q))
             (time (time-tag q))
             (action (car (params q))))
        
        (case dest
          (:cognition (case action
                        (select-production (when cognition
                                             (dolist (x (car (cadadr cognition)))
                                               (case (car x)
                                                 (=goal>
                                                  (setf last-goal (+ 10 (* 1000.0 time)))
                                                  )
                                                 (=retrieval>
                                                  (setf last-ret (+ 10 (* 1000.0 time))))
                                                 (=visual-location>
                                                  (setf last-vis-loc (+ 10 (* 1000.0 time))))
                                                 (=visual>
                                                  (setf last-vis (+ 10 (* 1000.0 time))))
                                                 ))
                                             (push (list 'rectangle (* 1000.0 time) 125 
                                                         (* 1000.0 (car cognition)) 155 (caadr cognition))
                                                   items)
                                             (setf cognition nil)
                                             ))
                        (execute-rhs (setf cognition (list time details))
                                     ;;; RHS actions
                                     (dolist (x (cdr (second details)))
                                       (let ((cmd (car x)))
                                         (case cmd
                                           ((=goal> +goal> -goal>)
                                            (push (list 'rectangle (* 1000.0 time) 75
                                                        (* 1000.0 time) 105 nil)
                                                  items)
                                            (push (list 'line (- (* 1000.0 time) 10) 125 (- (* 1000.0 time) 10) 90
                                                        (* 1000.0 time) 90) items)
                                            (when last-goal
                                                    (push (list 'line (* 1000.0 time) 90 last-goal 90 last-goal 125) items)))
                                           (+retrieval>
                                            (setf last-ret nil)
                                            (push (list 'line (- (* 1000.0 time) 10) 125 (- (* 1000.0 time) 10) 40
                                                        (* 1000.0 time) 40) items)
                                            (if retrieval
                                                (progn
                                                  (push (list 'rectangle (* 1000.0 time) 25
                                                              (* 1000.0 (car retrieval)) 55 (cadr retrieval))
                                                        items)
                                                  (setf retrieval nil))
                                                
                                                (if last-ret-request
                                                    
                                                    (push (list 'rectangle (* 1000.0 time) 25
                                                                (* 1000.0 last-ret-request) 55 'aborted)
                                                          items)
                                                    (push (list 'rectangle (* 1000.0 time) 25
                                                                (pm-get-time) 55 'unfinished)
                                                          items)))
                                            (setf last-ret-request time))
                                           (+visual-location>
                                            (setf last-vis-loc nil)
                                            (push (list 'line (- (* 1000.0 time) 10) 155 (- (* 1000.0 time) 10) 190
                                                        (* 1000.0 time) 190) items)) 
                                           ((-visual> +visual>)
                                            (setf last-vis nil)
                                            (push (list 'line (- (* 1000.0 time) 10) 155 (- (* 1000.0 time) 10) 240
                                                        (* 1000.0 time) 240) items))
                                           ((-manual> +manual>)
                                            (push (list 'line (- (* 1000.0 time) 10) 155 (- (* 1000.0 time) 10) 290
                                                        (* 1000.0 time) 290) items))
                                           )
                                         )))
                        
                        (complete-retrieval (unless retrieval
                                              (when last-ret
                                                (push (list 'line (* 1000.0 time) 40 last-ret 40 last-ret 125) items))
                                              (setf retrieval (list time (car details))))
                                            )))
          (:vision (case action
                     (find-location (when last-vis-loc
                                      (push (list 'line (* 1000.0 time) 190 last-vis-loc 190 last-vis-loc 155) items))
                                    (push (list 'rectangle (* 1000.0 time) 175
                                                            (* 1000.0 time) 205 nil)
                                                      items))
                     (move-attention 
                      (setf last-vis nil)
                      (if vision
                          (push (list 'rectangle (* 1000.0 time) 225
                                      (* 1000.0 (car vision)) 255 (cadr vision))
                                items)
                          (push (list 'rectangle (* 1000.0 time) 225
                                      (pm-get-time) 255 'unfinished)
                                items))
                      (setf vision nil))
                     (encoding-complete (setf vision (list time 'attend))
                                        (when last-vis
                                       (push (list 'line (* 1000.0 time) 240 last-vis 240 last-vis 155) items)))
                     (clear (if vision
                                           (push (list 'rectangle (* 1000.0 time) 225
                                                       (* 1000.0 (car vision)) 255 (cadr vision))
                                                 items)
                                           (push (list 'rectangle (* 1000.0 time) 225
                                                       (pm-get-time) 255 'unfinished)
                                                 items))
                                     (setf vision nil))
                     (change-state (setf vision (list time 'clear)))))
          (:motor (case action
                    (clear (if motor
                                           (push (list 'rectangle (* 1000.0 time) 275
                                                       (* 1000.0 (car motor)) 305 (cadr motor))
                                                 items)
                                           (push (list 'rectangle (* 1000.0 time) 275
                                                       (pm-get-time) 305 'unfinished)
                                                 items))
                           (setf motor nil))
                    (change-state (setf motor (list time 'clear)))
                    (press-key (if motor
                                             (push (list 'rectangle (* 1000.0 time) 275
                                                 (* 1000.0 (car motor)) 290 'press-key)
                                           items)
                                     (push (list 'rectangle (* 1000.0 time) 275
                                                 (pm-get-time) 290 'unfinished)
                                           items))
                                 (setf motor nil))
                    (preparation-complete (if motor
                                             (push (list 'rectangle (* 1000.0 time) 290
                                                 (* 1000.0 (car motor)) 305 (cadr motor))
                                           items)
                                     (push (list 'rectangle (* 1000.0 time) 290
                                                 (pm-get-time) 305 'unfinished)
                                           items))
                                 (setf motor (list time nil)))
                    (initiation-complete (if motor
                                             (progn
                                               (push (list 'rectangle (* 1000.0 time) 290
                                                           (* 1000.0 (car motor)) 305 (cadr motor))
                                                     items)
                                               (let ((startx (* 1000.0 (- (car motor) (/ (- (car motor) time) 4.0)))))
                                                 (push (list 'line startx 305 startx 340 (* 1000.0 (car motor)) 340) items)))
                                     (push (list 'rectangle (* 1000.0 time) 290
                                                 (pm-get-time) 305 'unfinished)
                                           items))
                                 (setf motor (list time 'init))
                                )
                    (finish-movement (setf motor (list time 'finish)))
                    ))
          (:device (case action
                     (output-key (push (list 'rectangle (* 1000.0 time) 325
                                                            (* 1000.0 time) 355 nil)
                                       items)
                                 (if motor
                                     (push (list 'rectangle (* 1000.0 time) 290
                                                 (* 1000.0 (car motor)) 305 (cadr motor))
                                           items)
                                     (push (list 'rectangle (* 1000.0 time) 290
                                                 (pm-get-time) 305 'unfinished)
                                           items))
                                 (setf motor (list time 'exec))
                                     
                                 )
                     )))
                    ))
                                                  
                         
                         
    items))

                                     



                         
#|
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
|#