;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Dan Bothell 
;;; Address     : Carnegie Mellon University
;;;             : Psychology Department
;;;             : Pittsburgh,PA 15213-3890
;;;             : db30+@andrew.cmu.edu
;;; 
;;; Copyright   : (c)2002-2004 Dan Bothell
;;; Availability: Covered by the GNU LGPL, see LGPL.txt
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : loader.lisp
;;; Version     : 1.5
;;; 
;;; Description : Mostly system dependent code.
;;;             : Just use a simple load file to start things up.
;;; Bugs        : 
;;; 
;;; Todo        : 
;;; 
;;; ----- History -----
;;;
;;; 05/30/2002  Dan
;;;             : Added this header and merged the ACL/MCL loaders
;;; 08/15/2002  Dan
;;;             : Added the LispWorks loader.
;;; 10/01/2002  Dan
;;;             : Updated version to 1.1 and fixed the packaging
;;;             : for building a standalone in ACL.
;;;             : Added standalone.lisp to the file list.
;;; 08/15/2003  Dan
;;;             : Updated to version 1.3
;;;             : Added the loader for CMUCL from Ethan Glasser-Camp at RPI.
;;; 4/22/2004   Dan [1.5]
;;;             : Removed the standalone.lisp from the list unless needed.
;;;             : Added the license info.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; uncomment this when building a standalone in ACL
;(pushnew :ACTR-ENV-ALONE *features*)


;(in-package :cl-user)

;#+(and :allegro-ide (not :ACTR-ENV-ALONE)) (in-package :cg-user)

;;; First, add an environment indicator to the features list
;;; so that I can test for it in the "experiment library" files
;;; of RPM when I move the UWI Tcl-side. 

(eval-when (:compile-toplevel :load-toplevel :execute)
  (pushnew :actr-environment *features*))


#+(and :allegro :actr-env-alone) (setf EXCL::*SOURCE-FILE-AUTOLOAD-SRECORD* nil)

;;; It doesn't like this if it's in uni-files where I'd prefer to have it...

#+:lispworks (require "comm")

;;; This needs to be here for standalone purposes

#+:allegro (require :sock)

;;; then load the ACT-R files

#+(or :lispworks :allegro :openmcl :cmu)
(load (merge-pathnames "ACT-R/+ load-rpm.lisp" *load-pathname*))

#+(and :mcl (not :openmcl))
(load (merge-pathnames ":ACT-R:+ load-rpm.lisp" *load-pathname*))

;;; Define a list of the files to load


#+(and :mcl (not :openmcl))
(defparameter *environment-file-list* '("hook-fns.lisp" 
                                        "mcl-fix.lisp"
                                        "uni-files.lisp" 
                                        "handlers.lisp"
                                        "server.lisp"
                                        "environment-cmds.lisp"
                                        "dm-stepper-hack.lisp"
                                        "stepper-control.lisp"
                                        "smart-loader.lisp"
                                        "parse-files.lisp"
                                        "env-device.lisp"
                                        "graphic-trace.lisp"
                                        ;"standalone.lisp"
                                        ))

#-(and :mcl (not :openmcl))
(defparameter *environment-file-list* '("hook-fns.lisp" 
                                        "uni-files.lisp" 
                                        "handlers.lisp"
                                        "server.lisp"
                                        "environment-cmds.lisp"
                                        "dm-stepper-hack.lisp"
                                        "stepper-control.lisp"
                                        "smart-loader.lisp"
                                        "parse-files.lisp"
                                        "env-device.lisp"
                                        "graphic-trace.lisp"
                                        #+:ACTR-ENV-ALONE "standalone.lisp"
                                        ))

;;; uni-compile-and-load
;;; This function takes one parameter which should be a string which
;;; names a file in the same directory as this file.  That file is loaded
;;; and compiled first if necessary (MCL systems compile as loading, so it's
;;; not really necessary to generate the fasl).  There is no test for
;;; date or version, so everything is always compiled fresh every time.
;;; Which doesn't seem like too big a deal since there aren't that many
;;; files and they aren't that big.

#+:allegro
(defun uni-compile-and-load (file)
  (compile-file (merge-pathnames file *load-pathname*) 
                :load-after-compile t))

#+:mcl
(defun uni-compile-and-load (file)
  (load (merge-pathnames file *load-pathname*)))

#+:cmu
(defun uni-compile-and-load (file)
  (compile-file (merge-pathnames file *load-pathname*) :load t))

#+(and :lispworks :win32)
(defun uni-compile-and-load (file)
  (let* ((srcpath (merge-pathnames file *load-pathname*))
         (binpath (merge-pathnames 
                     (make-pathname :name (pathname-name file) :type "fsl")
                     *load-pathname*)))
    (compile-file srcpath :output-file binpath)
    (load binpath)))


#+(and :lispworks :unix)
(defun uni-compile-and-load (file)
  (let* ((srcpath (merge-pathnames file *load-pathname*))
         (binpath (merge-pathnames 
                     (make-pathname :name (pathname-name file) :type "ufsl")
                     *load-pathname*)))
    (compile-file srcpath :output-file binpath)
    (load binpath)))

;;; Finally, just loop over the file list and load them

(dolist (x *environment-file-list*)
  (uni-compile-and-load x))

#|
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
|#