;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Dan Bothell 
;;; Address     : Carnegie Mellon University
;;;             : Psychology Department
;;;             : Pittsburgh,PA 15213-3890
;;;             : db30+@andrew.cmu.edu
;;; 
;;; Copyright   : (c)2002-2004 Dan Bothell
;;; Availability: Covered by the GNU LGPL, see LGPL.txt
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : standalone.lisp
;;; Version     : 1.5
;;; 
;;; Description : Contains the code for connecting an application
;;;             : version of the environment to the Tcl side.
;;;             : No system dependent code - at this point...
;;; Bugs        : 
;;; 
;;; Todo        :
;;; 
;;; ----- History -----
;;;
;;; 10/01/2002  Dan
;;;             : File creation
;;; 4/22/2004   Dan [1.5]
;;;             : Added the license info.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(in-package :cl-user)

;#+(and :allegro-ide (not :ACTR-ENV-ALONE)) (in-package :cg-user)

(defvar *global-output-stream*  (make-string-output-stream))
(defvar *stop-environment* nil)

(defun run-standalone ()
  (setf *chunk-type-file* (merge-pathnames "chunk-type.lisp" *install-dir*))
  (setf *chunk-file* (merge-pathnames "chunk.lisp" *install-dir*))
  (setf *production-file* (merge-pathnames "production.lisp" *install-dir*))
  (setf *misc-file* (merge-pathnames "misc.lisp" *install-dir*))
  (setf *command-file* (merge-pathnames "command.lisp" *install-dir*))
  
  (multiple-value-bind (s err) 
        (ignore-errors (uni-make-socket *tcl-address* *tcl-port*))
      (if (and (subtypep (type-of err) 'condition)
               (not (equal (type-of err) 'unbound-variable)))
          (uni-report-error err "Unable to Connect")
        (message-process s 1))))
                   

(defun eval-command (cmd)
  (let ((*standard-output* *global-output-stream*)
        (*error-output* *global-output-stream*))
    (format t "~%> ~A~%" cmd)
    (multiple-value-bind (result err)
        (ignore-errors (read-from-string cmd))
      (if (and (subtypep (type-of err) 'condition)
             (not (equal (type-of err) 'unbound-variable)))
        (progn
          (format t "~S~%" (type-of err))
          (uni-report-error 
           err (format nil 
                         "Error in command: ~s:~%"
                       cmd))
          
          nil)
        (multiple-value-bind (res err)
        (ignore-errors (eval result))
      (if (and (subtypep (type-of err) 'condition)
             (not (equal (type-of err) 'unbound-variable)))
        (progn
          (format t "~S~%" (type-of err))
          (uni-report-error 
           err (format nil 
                         "Error executing command: ~s:~%"
                       cmd))
          
          nil)
        (progn
          (format t "~%~S~%" res)
          nil)))))))

(defun start-listener-outputer (handler)
   (uni-run-process "Listener-outputer" #'(lambda () (run-listener-outputer handler)))
   "")

(defun run-listener-outputer (handler)
  (loop
    (uni-wait-for #'(lambda ()
                      (let ((text (get-output-stream-string *global-output-stream*)))
                        (if (> (length text) 0)
                            (progn
                              (setf (update-value handler) text)
                              t)
                          nil))))
    (send-update handler)))

#|
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
|#